@@ -5,6 +5,7 @@ import itertools
 import pprint
 import re
 import translationstring
+import warnings
 
 from .compat import (
     text_,
@@ -225,9 +226,13 @@ class Function(object):
     def __call__(self, node, value):
         result = self.function(value)
         if not result:
-            raise Invalid(node, translationstring.TranslationString(self.message, mapping={'val':value}))
+            raise Invalid(
+                node, translationstring.TranslationString(
+                    self.message, mapping={'val':value}))
         if isinstance(result, string_types):
-            raise Invalid(node, translationstring.TranslationString(result, mapping={'val':value}))
+            raise Invalid(
+                node, translationstring.TranslationString(
+                    result, mapping={'val':value}))
 
 class Regex(object):
     """ Regular expression validator.
@@ -351,6 +356,36 @@ class OneOf(object):
                     mapping={'val':value, 'choices':choices})
             raise Invalid(node, err)
 
+def luhnok(node, value):
+    """ Validator which checks to make sure that the value passes a luhn
+    mod-10 checksum (credit cards).  ``value`` must be a string, not an
+    integer."""
+    try:
+        sum = _luhnok(value)
+    except:
+        raise Invalid(node,
+                      '%r is not a valid credit card number' % value)
+
+    if not (sum % 10) == 0:
+        raise Invalid(node,
+                      '%r is not a valid credit card number' % value)
+
+def _luhnok(value):
+    sum = 0
+    num_digits = len(value)
+    oddeven = num_digits & 1
+
+    for count in range(0, num_digits):
+        digit = int(value[count])
+
+        if not (( count & 1 ) ^ oddeven ):
+            digit = digit * 2
+        if digit > 9:
+            digit = digit - 9
+
+        sum = sum + digit
+    return sum
+
 class SchemaType(object):
     """ Base class for all schema types """
     def flatten(self, node, appstruct, prefix='', listitem=False):
@@ -373,6 +408,9 @@ class SchemaType(object):
     def get_value(self, node, appstruct, path):
         raise AssertionError("Can't call 'get_value' on a leaf node.")
 
+    def cstruct_children(self, node, cstruct):
+        return []
+
 class Mapping(SchemaType):
     """ A type which represents a mapping of names to nodes.
 
@@ -443,6 +481,20 @@ class Mapping(SchemaType):
                           mapping = {'val':value, 'err':e})
                           )
 
+    def cstruct_children(self, node, cstruct):
+        if cstruct is null:
+            value = {}
+        else:
+            value = self._validate(node, cstruct)
+        children = []
+        for subnode in node.children:
+            name = subnode.name
+            subval = value.get(name, _marker)
+            if subval is _marker:
+                subval = subnode.serialize(null)
+            children.append(subval)
+        return children
+
     def _impl(self, node, value, callback):
         value = self._validate(node, value)
 
@@ -574,6 +626,22 @@ class Tuple(Positional, SchemaType):
 
         return list(value)
 
+    def cstruct_children(self, node, cstruct):
+        childlen = len(node.children)
+        if cstruct is null:
+            cstruct = []
+        structlen = len(cstruct)
+        if structlen < childlen:
+            missing_children = node.children[structlen:]
+            cstruct = list(cstruct)
+            for child in missing_children:
+                cstruct.append(child.serialize(null))
+        elif structlen > childlen:
+            cstruct = cstruct[:childlen]
+        else:
+            cstruct = list(cstruct)
+        return cstruct
+
     def _impl(self, node, value, callback):
         value = self._validate(node, value)
         error = None
@@ -665,6 +733,14 @@ class Tuple(Positional, SchemaType):
         return appstruct[index]
 
 
+class SequenceItems(list):
+    """
+    List marker subclass for use by Sequence.cstruct_children, which indicates
+    to a caller of that method that the result is from a sequence type.
+    Usually these values need to be treated specially, because all of the
+    children of a Sequence are not present in a schema.
+    """
+
 class Sequence(Positional, SchemaType):
     """
     A type which represents a variable-length sequence of nodes,
@@ -706,6 +782,11 @@ class Sequence(Positional, SchemaType):
                                   mapping={'val':value})
                           )
 
+    def cstruct_children(self, node, cstruct):
+        if cstruct is null:
+            return SequenceItems([])
+        return SequenceItems(cstruct)
+
     def _impl(self, node, value, callback, accept_scalar):
         if accept_scalar is None:
             accept_scalar = self.accept_scalar
@@ -947,7 +1028,7 @@ class Number(SchemaType):
     num = None
 
     def serialize(self, node, appstruct):
-        if appstruct is null:
+        if appstruct in (null, None):
             return null
 
         try:
@@ -996,8 +1077,42 @@ class Float(Number):
     num = float
 
 class Decimal(Number):
-    """ A type representing a decimal floating point.  Deserialization
-    returns an instance of the Python ``decimal.Decimal`` type.
+    """
+    A type representing a decimal floating point.  Deserialization returns an
+    instance of the Python ``decimal.Decimal`` type.
+
+    If the :attr:`colander.null` value is passed to the serialize
+    method of this class, the :attr:`colander.null` value will be
+    returned.
+
+    The Decimal constructor takes two optional arguments, ``quant`` and
+    ``rounding``.  If supplied, ``quant`` should be a string.  If supplied,
+    ``rounding`` should be one of the Python ``decimal`` module rounding
+    options (e.g. ``decimal.ROUND_UP``, ``decimal.ROUND_DOWN``, etc).  The
+    serialized and deserialized result will be quantized and rounded via
+    ``result.quantize(decimal.Decimal(quant), rounding)``.  ``rounding`` is
+    ignored if ``quant`` is not supplied.
+
+    The subnodes of the :class:`colander.SchemaNode` that wraps
+    this type are ignored.
+    """
+    def __init__(self, quant=None, rounding=None):
+        if quant is None:
+            self.quant = None
+        else:
+            self.quant = decimal.Decimal(quant)
+        self.rounding = rounding
+
+    def num(self, val):
+        result = decimal.Decimal(str(val))
+        if self.quant is not None:
+            result = result.quantize(self.quant, self.rounding)
+        return result
+
+class Money(Decimal):
+    """ A type representing a money value with two digit precision.
+    Deserialization returns an instance of the Python ``decimal.Decimal``
+    type (quantized to two decimal places, rounded up).
 
     If the :attr:`colander.null` value is passed to the serialize
     method of this class, the :attr:`colander.null` value will be
@@ -1006,8 +1121,9 @@ class Decimal(Number):
     The subnodes of the :class:`colander.SchemaNode` that wraps
     this type are ignored.
     """
-    def num(self, val):
-        return decimal.Decimal(str(val))
+    def __init__(self):
+        self.quant = decimal.Decimal('.01')
+        self.rounding = decimal.ROUND_UP
 
 class Boolean(SchemaType):
     """ A type representing a boolean object.
@@ -1464,7 +1580,7 @@ class SchemaNode(object):
       or one of the parents of the clone's prototype nodes.  The
       deepest nodes in the node tree are bound first, so the
       ``after_bind`` methods of the deepest nodes are called before
-      the shallowest.  The ``after_bind`` callback should should
+      the shallowest.  The ``after_bind`` callback should
       accept two values: ``node`` and ``kw``.  ``node`` will be a
       clone of the bound node object, ``kw`` will be the set of
       keywords passed to the ``bind`` method.
@@ -1598,7 +1714,13 @@ class SchemaNode(object):
         appstruct = self.typ.deserialize(self, cstruct)
 
         if self.preparer is not None:
-            appstruct = self.preparer(appstruct)
+            # if the preparer is a function, call a single preparer
+            if hasattr(self.preparer, '__call__'):
+                appstruct = self.preparer(appstruct)
+            # if the preparer is a list, call each separate preparer
+            elif is_nonstr_iter(self.preparer):
+                for preparer in self.preparer:
+                    appstruct = preparer(appstruct)
 
         if appstruct is null:
             appstruct = self.missing
@@ -1615,9 +1737,31 @@ class SchemaNode(object):
         return appstruct
 
     def add(self, node):
-        """ Add a subnode to this node. """
+        """ Append a subnode to this node. ``node`` must be a SchemaNode."""
         self.children.append(node)
 
+    def insert(self, index, node):
+        """ Insert a subnode into the position ``index``.  ``node`` must be
+        a SchemaNode."""
+        self.children.insert(index, node)
+
+    def add_before(self, name, node):
+        """ Insert a subnode into the position before the node named ``name``
+        """
+        for pos, sub in enumerate(self.children[:]):
+            if sub.name == name:
+                self.insert(pos, node)
+                return
+        raise KeyError('No such node named %s' % name)
+
+    def get(self, name, default=None):
+        """ Return the subnode associated with ``name`` or ``default`` if no
+        such node exists."""
+        for node in self.children:
+            if node.name == name:
+                return node
+        return default
+
     def clone(self):
         """ Clone the schema node and return the clone.  All subnodes
         are also cloned recursively.  Attributes present in node
@@ -1648,6 +1792,24 @@ class SchemaNode(object):
         if getattr(self, 'after_bind', None):
             self.after_bind(self, kw)
 
+    def cstruct_children(self, cstruct):
+        """ Will call the node's type's ``cstruct_children`` method with this
+        node as a first argument, and ``cstruct`` as a second argument."""
+        cstruct_children = getattr(self.typ, 'cstruct_children', None)
+        if cstruct_children is None:
+            warnings.warn(
+                'The node type %s has no cstruct_children method.'
+                'This method is required to be implemented by schema types for '
+                'compatibility with Colander 0.9.9+.  In a future Colander '
+                'version, the absence of this method will cause an '
+                'exception.  Returning [] for compatibility although it '
+                'may not be the right value.' % self.typ.__class__,
+                DeprecationWarning,
+                stacklevel=2
+                )
+            return []
+        return cstruct_children(self, cstruct)
+
     def __delitem__(self, name):
         """ Remove a subnode by name """
         for idx, node in enumerate(self.children[:]):
@@ -1657,30 +1819,31 @@ class SchemaNode(object):
 
     def __getitem__(self, name):
         """ Get a subnode by name. """
-        for node in self.children:
-            if node.name == name:
-                return node
-        raise KeyError(name)
+        val = self.get(name, _marker)
+        if val is _marker:
+            raise KeyError(name)
+        return val
 
     def __setitem__(self, name, newnode):
-        """ Replace a subnode by name """
+        """ Replace a subnode by name.  ``newnode`` must be a SchemaNode.  If
+        a subnode named ``name`` doesn't already exist, calling this method
+        is the same as setting the node's name to ``name`` and calling the
+        ``add`` method with the node (it will be appended to the children
+        list)."""
+        newnode.name = name
         for idx, node in enumerate(self.children[:]):
             if node.name == name:
                 self.children[idx] = newnode
-                newnode.name = name
                 return node
-        raise KeyError(name)
+        self.add(newnode)
 
     def __iter__(self):
         """ Iterate over the children nodes of this schema node """
         return iter(self.children)
 
     def __contains__(self, name):
-        try:
-            self[name]
-        except KeyError:
-            return False
-        return True
+        """ Return True if subnode named ``name`` exists in this node """
+        return self.get(name, _marker) is not _marker
 
     def __repr__(self):
         return '<%s.%s object at %d (named %s)>' % (
@@ -1717,10 +1880,23 @@ def _Schema__new__(cls, *args, **kw):
     typ = cls.schema_type()
     node.__init__(typ, *args, **kw)
     for n in cls.nodes:
-        node.add(n)
+        insert_before = getattr(n, 'insert_before', None)
+        exists = node.get(n.name, _marker) is not _marker
+        # use exists for microspeed; we could just call __setitem__
+        # exclusively, but it does an enumeration that's unnecessary in the
+        # common (nonexisting) case (.add is faster)
+        if insert_before is None:
+            if exists:
+                node[n.name] = n
+            else:
+                node.add(n)
+        else:
+            if exists:
+                del node[n.name]
+            node.add_before(insert_before, n)
     return node
 
-Schema = _SchemaMeta('Schema', (object,), 
+Schema = _SchemaMeta('Schema', (object,),
     dict(schema_type=Mapping,
         node_type=SchemaNode,
         __new__=_Schema__new__))
@@ -1728,7 +1904,7 @@ Schema = _SchemaMeta('Schema', (object,),
 MappingSchema = Schema
 
 
-def _SequanceSchema__new__(cls, *args, **kw):
+def _SequenceSchema__new__(cls, *args, **kw):
     node = object.__new__(cls.node_type)
     node.name = None
     node._order = next(SchemaNode._counter)
@@ -1744,7 +1920,7 @@ def _SequanceSchema__new__(cls, *args, **kw):
 SequenceSchema = _SchemaMeta('SequenceSchema', (object,),
     dict(schema_type=Sequence,
         node_type=SchemaNode,
-        __new__=_SequanceSchema__new__))
+        __new__=_SequenceSchema__new__))
 
 class TupleSchema(Schema):
     schema_type = Tuple
